import _ from 'lodash'
class System {
  constructor() {
    this.emptyList = [undefined, null]
    this.moduleName = 'SystemModule'
  }
  toParams(param) {
    var result = ''
    for (const name in param) {
      if (typeof param[name] !== 'function') {
        result += '&' + name + '=' + encodeURI(param[name])
      }
    }
    return result.substring(1)
  }
  getUrlParams() {
    const { db_name, is_mock, db_code } = this.getURLParameters()
    const { appCode, pageName } = this.parseAppCodePageName()
    const isMock = this.emptyList.includes(is_mock) ? 1 : is_mock
    const obj = _.cloneDeep({
      dbName: db_name || '',
      appCode,
      pageName,
      isMock,
      dbCode: db_code
    })
    return obj
  }
  getSysParams() {
    const { db_name, db_code } = this.getURLParameters()
    const {
      appCode,
      pageName
    } = this.parseAppCodePageName()
    const obj = _.cloneDeep({
      dbName: db_name,
      appCode,
      pageName: pageName,
      dbCode: db_code,
      pageCode: pageName
    })
    return obj
  }
  getURLParameters(url) {
    const url_ = url || location.href
    const obj = (url_.match(/([^?=&]+)(=([^&]*))/g) || []).reduce(
      (a, v) => ((a[v.slice(0, v.indexOf('='))] = v.slice(v.indexOf('=') + 1)), a),
      {}
    )
    for (const k in obj) {
      obj[k] = typeof obj[k] === 'string' ? decodeURIComponent(obj[k]) : obj[k]
    }
    return obj
  }
  parseGlobalParams() {
    const globalParam = {}
    let user_info = sessionStorage['user_info']
    if (user_info && user_info !== 'undefined') {
      user_info = JSON.parse(decodeURIComponent(user_info))
    }
    Object.assign(globalParam, user_info)
    return globalParam
  }
  parseAppCodePageName() {
    const hash = decodeURIComponent(location.hash)
    const hostSplit = hash.split('?')
    let target = hostSplit[0]
    // 如果说最后一位是‘/’ 则删除掉
    if (target.endsWith('/')) {
      target = target.substr(0, target.length - 1)
    }
    const targetSplit = target.split('/')
    const length = targetSplit.length
    let pageName = targetSplit[length - 1]
    const appCode = targetSplit[length - 2]
    if (pageName.includes('/')) {
      pageName = pageName.split('/')[0]
    }
    if (pageName.includes('?')) {
      pageName = pageName.split('?')[0]
    }
    return {
      appCode,
      pageName
    }
  }
  getClientHeight() {
    var clientHeight = 0
    if (document.body.clientHeight && document.documentElement.clientHeight) {
      clientHeight = (document.body.clientHeight < document.documentElement.clientHeight) ? document.body.clientHeight : document.documentElement.clientHeight
    } else {
      clientHeight = (document.body.clientHeight > document.documentElement.clientHeight) ? document.body.clientHeight : document.documentElement.clientHeight
    }
    return clientHeight
  }
  getClientWidth() {
    var clientWidth = 0
    if (document.body.clientWidth && document.documentElement.clientWidth) {
      clientWidth = (document.body.clientHeight < document.documentElement.clientWidth) ? document.body.clientWidth : document.documentElement.clientWidth
    } else {
      clientWidth = (document.body.clientWidth > document.documentElement.clientWidth) ? document.body.clientWidth : document.documentElement.clientWidth
    }
    return clientWidth
  }
}
export default new System()
